#ifndef	__FAP_SETUP_H__
#define  __FAP_SETUP_H__

#include "fap_macros.h"

/**
Specifies the number of frequency channels to be used for the frequency agility
protocol. The number of channels should be set as high as possible in order to 
increase the robustness against co-existing radio sources. However, a large number 
of frequency channels will increase the average transmission latency. 
*/
#define FAP_CH_TAB_SIZE 6                   

/**
Specifies the frequency channels to be used for the frequency agility
protocol. It must be spcified FAP_CH_TAB_SIZE number of cahnnels.
*/
#define FAP_CHANNELS {2, 15, 28, 42, 58, 73} //[channels]

/**
Specifies the time in [us] between each retransmit attempt for the device with the largest
payload size. Note that this value not equals the retransmit delay specified for 
the nrf radio, but the retransmit delay + radio on-air time + radio transmit startup
time.
*/
#define FAP_MAX_TX_PERIOD 437   // [us] Tx period NRF24L01 5 Byte PL
/**
Specifies the time in [us] between each retransmit attempt for the device with the smallest
payload size. Note that this value not equals the retransmit delay specified for 
the nrf radio, but the retransmit delay + radio on-air time + radio transmit startup
time.
*/
#define FAP_MIN_TX_PERIOD 437   // [us] Tx period NRF24L01 5 Byte PL

/**
Specifies the time in [us] the receiver listenst on each channel before channel switch.
*/
#define FAP_RX_PERIOD 754

/**
Specifies the number of auto retransmit attempts to be used for the nRF radio.
*/
#define FAP_NRF_AUTO_RETRIES 10         // For 6 channels

/**
The nRF radio has a limited number of auto retransmit attempts. This constant may
be used to extend the number of retransmit attempts. The total nummber of transmit 
attempts on a single channel before channel switch will equal 
(FAP_NRF_AUTO_RETRIES+1)*FAP_AUTO_RETRIES_EXTEND
*/
#define FAP_AUTO_RETRIES_EXTEND 2             // For 6 channels

/**
When receiving, the received data is placed in a FIFO consisting of the 
hardware (HW) FIFO provided in the nRF radio in addition to a FIFO contained in the
FAP firmware (FW). This constant specifies the number of FIFO levels for the 
FW FIFO. Thus, the total receive FIFO size will equal the number of
HW FIFO levels + FAP_FIFO_SIZE. The Nordic nRF24L01 provides 
a 3 level deep HW FIFO.
*/
#define FAP_FIFO_SIZE 2

/**
Specifies the 5 byte address to be used for device 0.
*/
#define FAP_ADDRESS_DEVICE0 {0xab,0x34,0x45,0xee,0x83} 

/**
Specifies the 5 byte address to be used for device 1.
*/       
#define FAP_ADDRESS_DEVICE1 {0x1a, 0xe7,0xbb,0xe7,0xfb}            

/**
Specifies byte 0 of the address to be used for device 2.
Address byte 1 to 4 will be set equal to byte 1 to 4 of
the adevice 1 address. 
*/       
#define FAP_ADDRESS_DEVICE2 0x2a  // Adr byte 1-4 = P1 byte 1 to 4

/**
Specifies byte 0 of the address to be used for device 3.
Address byte 1 to 4 will be set equal to byte 1 to 4 of
the adevice 1 address. 
*/
#define FAP_ADDRESS_DEVICE3 0x3a  // Adr byte 1-4 = P1 byte 1 to 4

/**
Specifies byte 0 of the address to be used for device 4.
Address byte 1 to 4 will be set equal to byte 1 to 4 of
the device 1 address. 
*/
#define FAP_ADDRESS_DEVICE4 0x4a  // Adr byte 1-4 = P1 byte 1 to 4

/**
Specifies byte 0 of the address to be used for device 5.
Address byte 1 to 4 will be set equal to byte 1 to 4 of
the device 1 address. 
*/
#define FAP_ADDRESS_DEVICE5 0x5a  // Adr byte 1-4 = P1 byte 1 to 4

/**
Specifies the payload length for device 0.
*/
#define FAP_PL_LENGTH_DEVICE0 5              // [bytes]

/**
Specifies the payload length for device 1.
*/
#define FAP_PL_LENGTH_DEVICE1 5              // [bytes]

/**
Specifies the payload length for device 2.
*/
#define FAP_PL_LENGTH_DEVICE2 5              // [bytes]

/**
Specifies the payload length for device 3.
*/
#define FAP_PL_LENGTH_DEVICE3 5              // [bytes]

/**
Specifies the payload length for device 4.
*/
#define FAP_PL_LENGTH_DEVICE4 5              // [bytes]

/**
Specifies the payload length for device 5.
*/
#define FAP_PL_LENGTH_DEVICE5 5              // [bytes]

/**
Specifies the maximum payload length in the system.
*/
#define FAP_MAX_PL_LENGTH 5                  // [bytes]

/**
Specifies the minimum payload length in the system.
*/
#define FAP_MIN_PL_LENGTH 5                  // [bytes]

/**
Speceifies the duration between each "receive burst" when using the
receive low power mode. This constants is specified in units of 
FAP_RX_PERIODs which in turn are speciefied in [us]. Thus, the duration
between each "receive burst" will equal 
FAP_LP_RX_POLL_PERIOD*FAP_RX_PERIOD [us]. 
*/
#define FAP_LP_RX_POLL_PERIOD (500000/FAP_RX_PERIOD)      

#endif

/**
@} @}
*/
