/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include <stdio.h>
#include <string.h>
#include "config.h"
#include "spi.h"
#include "nRF24L01.h"
#include "fap.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
GPIO_InitTypeDef GPIO_InitStructure;
ErrorStatus HSEStartUpStatus;
INT_HANDLER rf_interrupt;
INT_HANDLER timer_interrupt;
/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void NVIC_Configuration(void);
void Delay_ms( u16 time_ms );
void Delay_us( u8 time_us );
void RF_Interrupt_Init(void);
void RF_Timer_Init(u16 arr);
void LED_Output(u8 key_input);
u8 Get_Button(void);
void GPIO_Configuration(void);
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
  	u8 button=0x00;
	fap_rx_struct_t rx_data;				// Rx data
	u8 tx_data[5]={0x00};					// Tx payload buffer
	rf_interrupt.handler=NULL;
	timer_interrupt.handler=NULL;
	
  	RCC_Configuration();
	NVIC_Configuration();
	GPIO_Configuration();
	SPI1_Init();						// nRF24L01 SPI init
	USART_Configuration(CHANNEL_1);
	USART_Configuration(CHANNEL_2);

	RF_Interrupt_Init();					// RF interrupt init
	RF_Timer_Init(1008);					// RF timer init
	rf_interrupt.handler=fap_nrf_isr_function;
	timer_interrupt.handler=fap_timer_isr_function;
	
	fap_init();						// Frequency Agillity Protocol init
	fap_select_radio_idle_mode(FAP_PDOWN_IDLE);		// Set FAP Idle mode
	fap_rx_data(1,0);					
	
	myprintf(CHANNEL_1, "\r\nWIZ-RF30 F/W Ver.%d.%d",FW_VERSION_HIGH, FW_VERSION_LOW);
	
	LED_0_ON LED_1_ON LED_2_ON LED_3_ON LED_4_ON LED_5_ON LED_6_ON
	Delay_ms(30);
	LED_0_OFF LED_1_OFF LED_2_OFF LED_3_OFF LED_4_OFF LED_5_OFF LED_6_OFF
	   
	while(1)
	{
	  	button = Get_Button();
		//myprintf(CHANNEL_1, "\t 0x%0x", button); 
               if(button!=KEY_NONE)
		{
			LED_Output(button);			// light status led according to key_input
		   	tx_data[0]=button;			// Set button data to TX buffer
			button=0x00;				// init button
			CE_LOW();
			fap_tx_data(tx_data,1,500);		// Transmit Tx data to Pipe1, retransmit delay 500us
			while(fap_get_mode()!=FAP_IDLE){;}
			if(fap_get_tx_status()==FAP_TX_SUCCESS)
			{
			  	LED_0_ON
				Delay_ms(3);
				LED_0_OFF
			}
			else					// MAX retransmit interrupt
			{
			  	LED_0_ON
			}
			CE_LOW();
			fap_rx_data(1,0);
			CE_HIGH();
		}
	 	if(fap_read_rx_fifo(&rx_data))
		{
		   LED_0_ON
		   Delay_ms(10);
		   LED_0_OFF
		   LED_Output(rx_data.rx_pl[0]);		//output the status led according to key_input
		}
	}
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{   
  /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
 
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  }

  /* Enable GPIO_LED clock */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA|RCC_APB2Periph_GPIOB|RCC_APB2Periph_GPIOC|RCC_APB2Periph_SPI1|RCC_APB2Periph_USART1|RCC_APB2Periph_AFIO, ENABLE);
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2,ENABLE);
}
/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{
	  NVIC_InitTypeDef NVIC_InitStructure;
  /* Set the Vector Table base location at 0x2000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x4000);


  NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure); 
  
  NVIC_InitStructure.NVIC_IRQChannel = EXTI4_IRQChannel;;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
  
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

/*******************************************************************************
* Function Name  : GPIO_Configuration
* Description    : Configures GPIO port.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void GPIO_Configuration(void)
{
	u32 tmp;

	//  Disable the Serial Wire Jtag Debug Port SWJ-DP
    	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);
	tmp = AFIO->MAPR;
	AFIO->MAPR = tmp | 0x00002000;

        // Port A input
	GPIO_InitStructure.GPIO_Pin = L01_IRQ|BUTTON_3|BUTTON_6|BUTTON_5;   //@@@@semi
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_SetBits(GPIOA, L01_IRQ);   //@@@@semi
	GPIO_SetBits(GPIOA, BUTTON_3);   //@@@@semi
	GPIO_SetBits(GPIOA, BUTTON_6);   //@@@@semi

	GPIO_SetBits(GPIOA, BUTTON_5);   //@@@@semi
	GPIO_Init(GPIOA, &GPIO_InitStructure);
		
	// Port A output
	GPIO_InitStructure.GPIO_Pin = SPI_NSS2; 
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_SetBits(GPIOA, SPI_NSS2);

	// Configure the GPIO ports( USART1 Transmit and Receive Lines)
	//  Configure the USART1_Tx as Alternate function Push-Pull 
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Pin =  USART1_TX |USART1_RTS |USART2_TX |USART2_RTS;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// Configure the USART1_Rx as input floating
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Pin = USART1_RX |USART1_CTS | USART2_RX |USART2_CTS;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// SPI 1
	/* Configure SPIy pins: SCK, MISO and MOSI */
	GPIO_InitStructure.GPIO_Pin = WIZ_SCLK | WIZ_MISO | WIZ_MOSI;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
    
	// Port B
	// In/out
	//    GPIO_InitStructure.GPIO_Pin = GPIO_0|GPIO_1|GPIO_2; 
	GPIO_InitStructure.GPIO_Pin = GPIO_2; 
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_SetBits(GPIOB, GPIO_2);
	
	// Port B input
	//    GPIO_InitStructure.GPIO_Pin = LINK_INPUT|RXLED_INPUT|TXLED_INPUT
	GPIO_InitStructure.GPIO_Pin = BUTTON_1|BUTTON_2|BUTTON_4;  //@@@@semi
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_SetBits(GPIOB, BUTTON_1);
	GPIO_SetBits(GPIOB, BUTTON_2);
	GPIO_SetBits(GPIOB, BUTTON_4);
	
	// Port B output
	GPIO_InitStructure.GPIO_Pin = TEST_LED_0|nACT_LED|SPI_NSS1|L01_CE|L01_CSN|TEST_LED_4|TEST_LED_5|TEST_LED_6; //@@@@semi 
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_SetBits(GPIOB, TEST_LED_0);
	GPIO_SetBits(GPIOB, nACT_LED);
	GPIO_SetBits(GPIOB, SPI_NSS1);
	GPIO_SetBits(GPIOB, TEST_LED_4);
	GPIO_SetBits(GPIOB, TEST_LED_5);
	GPIO_SetBits(GPIOB, TEST_LED_6);
		    	
	// SPI 2
	/* Configure SPIy pins: SCK, MISO and MOSI */
	GPIO_InitStructure.GPIO_Pin = SPI_SCLK | SPI_MISO| SPI_MOSI;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
		
	// Port C Output
	//    GPIO_InitStructure.GPIO_Pin = nRESET_W5100 | USART1_STATUS|USART2_STATUS;
	GPIO_InitStructure.GPIO_Pin = TEST_LED_1|TEST_LED_2|TEST_LED_3;	//@@@@semi
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_SetBits(GPIOC, TEST_LED_1);	
	GPIO_SetBits(GPIOC, TEST_LED_2);	
	GPIO_SetBits(GPIOC, TEST_LED_3);	
}

/*******************************************************************************
* Function Name  : Get_Button
* Description    : Parse inputted button value.
* Input          : None
* Output         : None
* Return         : Key data
*******************************************************************************/
u8 key_flag=0;

u8 Get_Button(void) 
{
	u8	key;
	key = KEY_INPUT;
	if(key == KEY_NONE)
	{
	   if(!key_flag)
	      return key;
	   else
	   {
	      Delay_ms(20);
	      key_flag = 0;
	      return key;
	   }
	}
	else
	{
	   if(key_flag)
	      return KEY_NONE;
	   else
	   {
	      key_flag = 1;
	      return key;
	   }
	}
}

/*******************************************************************************
* Function Name  : RF_Interrupt_Init
* Description    : Initialize RF interrupt parameter.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RF_Interrupt_Init(void)
{
	EXTI_InitTypeDef EXTI_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	EXTI_DeInit();

	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA,GPIO_PinSource4);
	EXTI_InitStructure.EXTI_Line = EXTI_Line4;
	EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&EXTI_InitStructure);
	EXTI_GenerateSWInterrupt(EXTI_Line4);
	EXTI->IMR|=(1<<4);
	EXTI->EMR|=(1<<4);
}

/*******************************************************************************
* Function Name  : RF_Timer_Init
* Description    : Initialize RF timer interrupt parameter.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RF_Timer_Init(u16 arr)
{
  	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
	
	TIM_TimeBaseStructure.TIM_Period = arr;
  	TIM_TimeBaseStructure.TIM_Prescaler = 72;
  	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
	
	TIM2->CR1|=0x0040;
	
	TIM_ITConfig(TIM2,TIM_IT_Update,ENABLE);
	TIM_Cmd(TIM2,ENABLE);	
}

/*******************************************************************************
* Function Name  : LED_Output
* Description    : Set LED according to key_input
* Input          : key_input
* Output         : None
* Return         : None
*******************************************************************************/
void LED_Output(u8 key_input)
{

	if((key_input|0x1F)!=0x3F)
	 	TOGGLE_LED_1
	if((key_input|0x2F)!=0x3F)
		TOGGLE_LED_2
	if((key_input|0x37)!=0x3F)
	 	TOGGLE_LED_3
	if((key_input|0x3B)!=0x3F)
	 	TOGGLE_LED_4
	if((key_input|0x3D)!=0x3F)
	 	TOGGLE_LED_5
	if((key_input|0x3E)!=0x3F)
	 	TOGGLE_LED_6
}

/*******************************************************************************
* Function Name  : Delay_us
* Description    : Delay per micro second.
* Input          : time_us
* Output         : None
* Return         : None
*******************************************************************************/
void Delay_us( u8 time_us )
{
	register u8 i;
        register u8 j;
	for( i=0;i<time_us;i++ )	
	{
          for( j=0;j<5;j++ )	//25CLK
	  {
		asm("nop");	//1CLK         
		asm("nop");	//1CLK         
		asm("nop");	//1CLK         
		asm("nop");	//1CLK         
      		asm("nop");	//1CLK                  
                
          }      
	}			//25CLK*0.04us=1us
}

/*******************************************************************************
* Function Name  : Delay_ms
* Description    : Delay per mili second.
* Input          : time_ms
* Output         : None
* Return         : None
*******************************************************************************/
void Delay_ms( u16 time_ms )
{
	register u16 i;
	for( i=0;i<time_ms;i++ )
	{
		Delay_us(250);
		Delay_us(250);
		Delay_us(250);
		Delay_us(250);
	}
}
