#include "stm32f10x_lib.h"
#include "config.h"
#include "util.h"
#include <stdio.h>
#include <stdarg.h>

#define __GNUC__

#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */

static vu32 TimingDelay;


void USART_Configuration(u8 ch)
{
	
  USART_InitTypeDef USART_InitStructure;
  USART_ClockInitTypeDef USART_ClockInitStruct;

	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No ;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	 USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;

	  USART_ClockInitStruct.USART_Clock = USART_Clock_Disable;
	  USART_ClockInitStruct.USART_CPOL = USART_CPOL_Low;
	  USART_ClockInitStruct.USART_CPHA = USART_CPHA_2Edge;
	  USART_ClockInitStruct.USART_LastBit = USART_LastBit_Disable;


  if (ch == CHANNEL_1) {
//	  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
	  USART_ClockInit(USART1, &USART_ClockInitStruct);
	  USART_Init(USART1, &USART_InitStructure);
	  USART_Cmd(USART1, ENABLE);
  }else if (ch == CHANNEL_2) {
//	  USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
	  USART_ClockInit(USART2, &USART_ClockInitStruct);
	  USART_Init(USART2, &USART_InitStructure);
	  USART_Cmd(USART2, ENABLE);
   }

}


void myprintf(u8 ch, char *fmt, ...)
{
	va_list arg_ptr;
	char etxt[128];
	u8 i;
	USART_TypeDef* USARTx;
	
	if (ch == CHANNEL_1) USARTx = USART1;
	else if (ch == CHANNEL_2) USARTx = USART2;
	
	va_start(arg_ptr, fmt);
	vsprintf(etxt, fmt, arg_ptr);
	va_end(arg_ptr);

	for (i = 0; etxt[i] != '\0'; i++) {
		USART_SendData(USARTx, (u8) etxt[i]);
		while(USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == RESET); // Tx buffer empty
	}
}


void PutByte(u8 ch, u8 data)
{

	USART_TypeDef* USARTx;

	if (ch == CHANNEL_1) USARTx = USART1;
	else if (ch == CHANNEL_2) USARTx = USART2;
	
	USART_SendData(USARTx, (u8) data);
	while(USART_GetFlagStatus(USARTx, USART_FLAG_TXE) == RESET); // Tx buffer empty

	  
    	while(USART_GetFlagStatus(USARTx, USART_FLAG_TC) == RESET);
	USART_ClearFlag(USARTx, USART_FLAG_TC);
  
}


void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length.
* Output         : None
* Return         : None
*******************************************************************************/
// 4 usec delay
void Delay(u32 nCount)
{

#ifdef ENABLE_TIMER_INT

  TIM_Cmd(TIM3, ENABLE);
  
  TimingDelay = nCount;

  while(TimingDelay != 0);

  TIM_Cmd(TIM3, DISABLE);


#else
	nCount = nCount*1000;
  for(; nCount != 0; nCount--);

#endif

  
}


void wait_10ms(u32 cnt)
{
	u8 i;
	
	for(i=0; i < 10; i++) wait_1ms(cnt);

}

void wait_1ms(u32 cnt)
{
	u8 i;
	
	for(i=0; i < 250; i++) wait_4us(cnt);
}

void wait_4us(u32 cnt)
{
	u16 i;
	for (i = 0; i < cnt; i++) Delay(1);

}


/*******************************************************************************
* Function Name  : PUTCHAR_PROTOTYPE
* Description    : Retargets the C library printf function to the USART.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
/*
PUTCHAR_PROTOTYPE
{
  // Write a character to the USART
  USART_SendData(USART1, (u8) ch);

  //  Loop until the end of transmission 
  while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
  {
  }

  return ch;
}
*/

int putchar(int ch)
{
  // Write a character to the USART
  USART_SendData(USART1, (u8) ch);

  //  Loop until the end of transmission 
  while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)
  {
  }

  return ch;
}

u8 Get_Char(void)
{
u16 ch;

	while(USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET){
	}

	ch = USART_ReceiveData(USART1);

return ch;
}


#if 0
/*******************************************************************************
* Function Name  : USART_Scanf
* Description    : Gets numeric values from the hyperterminal.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
u8 USART_Scanf(u32 value)
{
  u32 index = 0;
  u32 tmp[2] = {0, 0};

  while(index < 2)
  {
    /* Loop until RXNE = 1 */
    while(USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET);
	
    tmp[index++] = (USART_ReceiveData(USART1));

    if((tmp[index - 1] < 0x30) || (tmp[index - 1] > 0x39))
    {
      printf("\n\rPlease enter valid number between 0 and 9");
      index--;
    }
  }
  /* Calculate the Corresponding value */
  index = (tmp[1] - 0x30) + ((tmp[0] - 0x30) * 10);
  /* Checks */
  if(index > value)
  {
    printf("\n\rPlease enter valid number between 0 and %d", value);
    return 0xFF;
  }
  return index;
}
#endif

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif



